@@ -24,6 +24,20 @@ COSE_KEY_TYPES = {
     "WalnutDSA": 6,  # WalnutDSA public key
 }
 
+# COSE key operation values.
+COSE_KEY_OPERATION_VALUES = {
+    "sign": 1,
+    "verify": 2,
+    "encrypt": 3,
+    "decrypt": 4,
+    "wrap key": 5,
+    "unwrap key": 6,
+    "derive key": 7,
+    "derive bits": 8,
+    "MAC create": 9,
+    "MAC verify": 10,
+}
+
 # COSE Algorithms for Content Encryption Key (CEK).
 COSE_ALGORITHMS_CEK = {
     "A128GCM": 1,  # AES-GCM mode w/ 128-bit key, 128-bit tag
@@ -33,6 +47,7 @@ COSE_ALGORITHMS_CEK = {
     "AES-CCM-16-64-256": 11,  # AES-CCM mode 256-bit key, 64-bit tag, 13-byte nonce
     "AES-CCM-64-64-128": 12,  # AES-CCM mode 128-bit key, 64-bit tag, 7-byte nonce
     "AES-CCM-64-64-256": 13,  # AES-CCM mode 256-bit key, 64-bit tag, 7-byte nonce
+    "ChaCha20/Poly1305": 24,  # ChaCha20/Poly1305 w/ 256-bit key, 128-bit tag
     "AES-CCM-16-128-128": 30,  # AES-CCM mode 128-bit key, 128-bit tag, 13-byte nonce
     "AES-CCM-16-128-256": 31,  # AES-CCM mode 256-bit key, 128-bit tag, 13-byte nonce
     "AES-CCM-64-128-128": 32,  # AES-CCM mode 128-bit key, 128-bit tag, 7-byte nonce
