@@ -22,13 +22,15 @@ class TestCOSEKey:
     """
 
     def test_cose_key_constructor(self):
-        """"""
         key = COSEKey({1: 1, 2: b"123", 3: 1})
         assert key.kty == 1
         assert key.kid == b"123"
         assert key.alg == 1
         assert key.key_ops is None
         assert key.base_iv is None
+        with pytest.raises(NotImplementedError):
+            key.generate_nonce()
+            pytest.fail("COSEKey.generate_nonce() should fail.")
         with pytest.raises(NotImplementedError):
             key.sign(b"message")
             pytest.fail("COSEKey.sign() should fail.")
@@ -43,7 +45,6 @@ class TestCOSEKey:
             pytest.fail("COSEKey.decrypt() should fail.")
 
     def test_cose_key_constructor_without_cose_key(self):
-        """"""
         with pytest.raises(TypeError):
             COSEKey()
             pytest.fail("COSEKey should fail.")
@@ -118,7 +119,6 @@ class TestCOSEKey:
         ],
     )
     def test_cose_key_constructor_with_invalid_args(self, invalid, msg):
-        """"""
         with pytest.raises(ValueError) as err:
             COSEKey(invalid)
             pytest.fail("COSEKey should fail.")
