@@ -13,7 +13,6 @@ class TestEC2Key:
     """
 
     def test_ec2_key_constructor_with_es256_key(self):
-        """"""
         private_key = EC2Key(
             {
                 1: 2,
@@ -36,12 +35,15 @@ class TestEC2Key:
         assert private_key.kty == 2
         assert private_key.kid is None
         assert private_key.alg == 1
-        assert private_key.key_ops is None
+        assert len(private_key.key_ops) == 2
+        assert 1 in private_key.key_ops
+        assert 2 in private_key.key_ops
         assert private_key.base_iv is None
         assert public_key.kty == 2
         assert public_key.kid is None
         assert public_key.alg == 1
-        assert public_key.key_ops is None
+        assert len(public_key.key_ops) == 1
+        assert 2 in public_key.key_ops
         assert public_key.base_iv is None
         try:
             sig = private_key.sign(b"Hello world!")
@@ -50,7 +52,6 @@ class TestEC2Key:
             pytest.fail("sign/verify should not fail.")
 
     def test_ec2_key_constructor_with_es384_key(self):
-        """"""
         private_key = EC2Key(
             {
                 1: 2,
@@ -73,12 +74,15 @@ class TestEC2Key:
         assert private_key.kty == 2
         assert private_key.kid is None
         assert private_key.alg == 2
-        assert private_key.key_ops is None
+        assert len(private_key.key_ops) == 2
+        assert 1 in private_key.key_ops
+        assert 2 in private_key.key_ops
         assert private_key.base_iv is None
         assert public_key.kty == 2
         assert public_key.kid is None
         assert public_key.alg == 2
-        assert public_key.key_ops is None
+        assert len(public_key.key_ops) == 1
+        assert 2 in public_key.key_ops
         assert public_key.base_iv is None
         try:
             sig = private_key.sign(b"Hello world!")
@@ -87,7 +91,6 @@ class TestEC2Key:
             pytest.fail("sign/verify should not fail.")
 
     def test_ec2_key_constructor_with_es512_key(self):
-        """"""
         private_key = EC2Key(
             {
                 1: 2,
@@ -110,12 +113,15 @@ class TestEC2Key:
         assert private_key.kty == 2
         assert private_key.kid is None
         assert private_key.alg == 3
-        assert private_key.key_ops is None
+        assert len(private_key.key_ops) == 2
+        assert 1 in private_key.key_ops
+        assert 2 in private_key.key_ops
         assert private_key.base_iv is None
         assert public_key.kty == 2
         assert public_key.kid is None
         assert public_key.alg == 3
-        assert public_key.key_ops is None
+        assert len(public_key.key_ops) == 1
+        assert 2 in public_key.key_ops
         assert public_key.base_iv is None
         try:
             sig = private_key.sign(b"Hello world!")
@@ -124,7 +130,6 @@ class TestEC2Key:
             pytest.fail("sign/verify should not fail.")
 
     def test_ec2_(self):
-        """"""
         private_key = EC2Key(
             {
                 1: 2,
@@ -147,12 +152,15 @@ class TestEC2Key:
         assert private_key.kty == 2
         assert private_key.kid is None
         assert private_key.alg == 3
-        assert private_key.key_ops is None
+        assert len(private_key.key_ops) == 2
+        assert 1 in private_key.key_ops
+        assert 2 in private_key.key_ops
         assert private_key.base_iv is None
         assert public_key.kty == 2
         assert public_key.kid is None
         assert public_key.alg == 3
-        assert public_key.key_ops is None
+        assert len(public_key.key_ops) == 1
+        assert 2 in public_key.key_ops
         assert public_key.base_iv is None
         try:
             sig = private_key.sign(b"Hello world!")
@@ -161,7 +169,6 @@ class TestEC2Key:
             pytest.fail("sign/verify should not fail.")
 
     def test_cose_key_constructor_without_cose_key(self):
-        """"""
         with pytest.raises(TypeError):
             EC2Key()
             pytest.fail("EC2Key should fail.")
@@ -372,14 +379,12 @@ class TestEC2Key:
         ],
     )
     def test_ec2_key_constructor_with_invalid_args(self, invalid, msg):
-        """"""
         with pytest.raises(ValueError) as err:
             EC2Key(invalid)
             pytest.fail("EC2Key should fail.")
         assert msg in str(err.value)
 
     def test_ec2_key_sign_with_es256_public_key(self):
-        """"""
         public_key = EC2Key(
             {
                 1: 2,
@@ -395,7 +400,6 @@ class TestEC2Key:
         assert "Public key cannot be used for signing." in str(err.value)
 
     def test_ec2_key_verify_with_another_es256_public_key(self):
-        """"""
         private_key = EC2Key(
             {
                 1: 2,
@@ -423,7 +427,6 @@ class TestEC2Key:
         assert "Failed to verify." in str(err.value)
 
     def test_ec2_key_verify_with_invalid_signature(self):
-        """"""
         private_key = EC2Key(
             {
                 1: 2,
