@@ -13,7 +13,6 @@ class TestOKPKey:
     """
 
     def test_okp_key_constructor_with_ed25519_key(self):
-        """"""
         private_key = OKPKey(
             {
                 1: 1,
@@ -34,12 +33,15 @@ class TestOKPKey:
         assert private_key.kty == 1
         assert private_key.kid is None
         assert private_key.alg == -8
-        assert private_key.key_ops is None
+        assert len(private_key.key_ops) == 2
+        assert 1 in private_key.key_ops
+        assert 2 in private_key.key_ops
         assert private_key.base_iv is None
         assert public_key.kty == 1
         assert public_key.kid is None
         assert public_key.alg == -8
-        assert public_key.key_ops is None
+        assert len(public_key.key_ops) == 1
+        assert 2 in public_key.key_ops
         assert public_key.base_iv is None
         try:
             sig = private_key.sign(b"Hello world!")
@@ -181,14 +183,12 @@ class TestOKPKey:
         ],
     )
     def test_okp_key_constructor_with_invalid_args(self, invalid, msg):
-        """"""
         with pytest.raises(ValueError) as err:
             OKPKey(invalid)
             pytest.fail("OKPKey should fail.")
         assert msg in str(err.value)
 
     def test_okp_key_sign_with_es256_public_key(self):
-        """"""
         public_key = OKPKey(
             {
                 1: 1,
@@ -203,7 +203,6 @@ class TestOKPKey:
         assert "Public key cannot be used for signing." in str(err.value)
 
     def test_okp_key_verify_with_invalid_signature(self):
-        """"""
         private_key = OKPKey(
             {
                 1: 1,
